#include "pch.h"
#include <iostream>
#include <boost/version.hpp>
namespace mongo { const char * gitVersion(){ return "a1810524d6ecd22250b97369c58a9d2bd090702e"; } }
namespace mongo { string sysInfo(){ return "Linux ip-10-110-9-236 2.6.21.7-2.ec2.v1.2.fc8xen #1 SMP Fri Nov 20 17:48:28 EST 2009 x86_64 BOOST_LIB_VERSION=" BOOST_LIB_VERSION ; } }

